# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'preferences.ui'
#
# Created by: PyQt5 UI code generator 5.15.6
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_prefDialog(object):
    def setupUi(self, prefDialog):
        prefDialog.setObjectName("prefDialog")
        prefDialog.setWindowModality(QtCore.Qt.WindowModal)
        prefDialog.resize(719, 554)
        self.gridLayout = QtWidgets.QGridLayout(prefDialog)
        self.gridLayout.setObjectName("gridLayout")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout()
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.tabWidget = QtWidgets.QTabWidget(prefDialog)
        self.tabWidget.setEnabled(True)
        self.tabWidget.setObjectName("tabWidget")
        self.tab_project = QtWidgets.QWidget()
        self.tab_project.setObjectName("tab_project")
        self.verticalLayout_5 = QtWidgets.QVBoxLayout(self.tab_project)
        self.verticalLayout_5.setObjectName("verticalLayout_5")
        self.horizontalLayout_14 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_14.setObjectName("horizontalLayout_14")
        self.label = QtWidgets.QLabel(self.tab_project)
        self.label.setObjectName("label")
        self.horizontalLayout_14.addWidget(self.label)
        self.cbTimeFormat = QtWidgets.QComboBox(self.tab_project)
        self.cbTimeFormat.setObjectName("cbTimeFormat")
        self.cbTimeFormat.addItem("")
        self.cbTimeFormat.addItem("")
        self.horizontalLayout_14.addWidget(self.cbTimeFormat)
        self.verticalLayout_5.addLayout(self.horizontalLayout_14)
        self.horizontalLayout_15 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_15.setObjectName("horizontalLayout_15")
        self.label_6 = QtWidgets.QLabel(self.tab_project)
        self.label_6.setObjectName("label_6")
        self.horizontalLayout_15.addWidget(self.label_6)
        self.sbAutomaticBackup = QtWidgets.QSpinBox(self.tab_project)
        self.sbAutomaticBackup.setMinimum(-10000)
        self.sbAutomaticBackup.setMaximum(10000)
        self.sbAutomaticBackup.setProperty("value", 10)
        self.sbAutomaticBackup.setObjectName("sbAutomaticBackup")
        self.horizontalLayout_15.addWidget(self.sbAutomaticBackup)
        self.verticalLayout_5.addLayout(self.horizontalLayout_15)
        self.horizontalLayout_13 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_13.setObjectName("horizontalLayout_13")
        self.label_3 = QtWidgets.QLabel(self.tab_project)
        self.label_3.setObjectName("label_3")
        self.horizontalLayout_13.addWidget(self.label_3)
        self.leSeparator = QtWidgets.QLineEdit(self.tab_project)
        self.leSeparator.setObjectName("leSeparator")
        self.horizontalLayout_13.addWidget(self.leSeparator)
        self.verticalLayout_5.addLayout(self.horizontalLayout_13)
        self.cbCheckForNewVersion = QtWidgets.QCheckBox(self.tab_project)
        self.cbCheckForNewVersion.setObjectName("cbCheckForNewVersion")
        self.verticalLayout_5.addWidget(self.cbCheckForNewVersion)
        self.horizontalLayout_11 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_11.setObjectName("horizontalLayout_11")
        self.lb_hwdec = QtWidgets.QLabel(self.tab_project)
        self.lb_hwdec.setObjectName("lb_hwdec")
        self.horizontalLayout_11.addWidget(self.lb_hwdec)
        self.cb_hwdec = QtWidgets.QComboBox(self.tab_project)
        self.cb_hwdec.setObjectName("cb_hwdec")
        self.horizontalLayout_11.addWidget(self.cb_hwdec)
        self.verticalLayout_5.addLayout(self.horizontalLayout_11)
        self.horizontalLayout_9 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_9.setObjectName("horizontalLayout_9")
        self.lb_project_file_indent = QtWidgets.QLabel(self.tab_project)
        self.lb_project_file_indent.setObjectName("lb_project_file_indent")
        self.horizontalLayout_9.addWidget(self.lb_project_file_indent)
        self.combo_project_file_indentation = QtWidgets.QComboBox(self.tab_project)
        self.combo_project_file_indentation.setObjectName("combo_project_file_indentation")
        self.horizontalLayout_9.addWidget(self.combo_project_file_indentation)
        self.verticalLayout_5.addLayout(self.horizontalLayout_9)
        spacerItem = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_5.addItem(spacerItem)
        self.tabWidget.addTab(self.tab_project, "")
        self.tab_observations = QtWidgets.QWidget()
        self.tab_observations.setObjectName("tab_observations")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.tab_observations)
        self.verticalLayout.setObjectName("verticalLayout")
        self.horizontalLayout_4 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_4.setObjectName("horizontalLayout_4")
        self.label_4 = QtWidgets.QLabel(self.tab_observations)
        self.label_4.setObjectName("label_4")
        self.horizontalLayout_4.addWidget(self.label_4)
        self.sbffSpeed = QtWidgets.QSpinBox(self.tab_observations)
        self.sbffSpeed.setMinimum(0)
        self.sbffSpeed.setMaximum(10000)
        self.sbffSpeed.setProperty("value", 10)
        self.sbffSpeed.setObjectName("sbffSpeed")
        self.horizontalLayout_4.addWidget(self.sbffSpeed)
        self.verticalLayout.addLayout(self.horizontalLayout_4)
        self.cb_adapt_fast_jump = QtWidgets.QCheckBox(self.tab_observations)
        self.cb_adapt_fast_jump.setObjectName("cb_adapt_fast_jump")
        self.verticalLayout.addWidget(self.cb_adapt_fast_jump)
        self.horizontalLayout_5 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_5.setObjectName("horizontalLayout_5")
        self.label_5 = QtWidgets.QLabel(self.tab_observations)
        self.label_5.setObjectName("label_5")
        self.horizontalLayout_5.addWidget(self.label_5)
        self.sbSpeedStep = QtWidgets.QDoubleSpinBox(self.tab_observations)
        self.sbSpeedStep.setDecimals(1)
        self.sbSpeedStep.setMinimum(0.1)
        self.sbSpeedStep.setMaximum(10.0)
        self.sbSpeedStep.setSingleStep(0.1)
        self.sbSpeedStep.setProperty("value", 0.1)
        self.sbSpeedStep.setObjectName("sbSpeedStep")
        self.horizontalLayout_5.addWidget(self.sbSpeedStep)
        self.verticalLayout.addLayout(self.horizontalLayout_5)
        self.horizontalLayout_6 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_6.setObjectName("horizontalLayout_6")
        self.label_2 = QtWidgets.QLabel(self.tab_observations)
        self.label_2.setObjectName("label_2")
        self.horizontalLayout_6.addWidget(self.label_2)
        self.sbRepositionTimeOffset = QtWidgets.QSpinBox(self.tab_observations)
        self.sbRepositionTimeOffset.setMinimum(-10000)
        self.sbRepositionTimeOffset.setMaximum(10000)
        self.sbRepositionTimeOffset.setProperty("value", -3)
        self.sbRepositionTimeOffset.setObjectName("sbRepositionTimeOffset")
        self.horizontalLayout_6.addWidget(self.sbRepositionTimeOffset)
        self.verticalLayout.addLayout(self.horizontalLayout_6)
        self.cbConfirmSound = QtWidgets.QCheckBox(self.tab_observations)
        self.cbConfirmSound.setObjectName("cbConfirmSound")
        self.verticalLayout.addWidget(self.cbConfirmSound)
        self.cbCloseSameEvent = QtWidgets.QCheckBox(self.tab_observations)
        self.cbCloseSameEvent.setObjectName("cbCloseSameEvent")
        self.verticalLayout.addWidget(self.cbCloseSameEvent)
        self.horizontalLayout_8 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_8.setObjectName("horizontalLayout_8")
        self.label_8 = QtWidgets.QLabel(self.tab_observations)
        self.label_8.setObjectName("label_8")
        self.horizontalLayout_8.addWidget(self.label_8)
        self.sbBeepEvery = QtWidgets.QSpinBox(self.tab_observations)
        self.sbBeepEvery.setObjectName("sbBeepEvery")
        self.horizontalLayout_8.addWidget(self.sbBeepEvery)
        self.verticalLayout.addLayout(self.horizontalLayout_8)
        self.cb_display_subtitles = QtWidgets.QCheckBox(self.tab_observations)
        self.cb_display_subtitles.setObjectName("cb_display_subtitles")
        self.verticalLayout.addWidget(self.cb_display_subtitles)
        self.cbTrackingCursorAboveEvent = QtWidgets.QCheckBox(self.tab_observations)
        self.cbTrackingCursorAboveEvent.setObjectName("cbTrackingCursorAboveEvent")
        self.verticalLayout.addWidget(self.cbTrackingCursorAboveEvent)
        self.cbAlertNoFocalSubject = QtWidgets.QCheckBox(self.tab_observations)
        self.cbAlertNoFocalSubject.setObjectName("cbAlertNoFocalSubject")
        self.verticalLayout.addWidget(self.cbAlertNoFocalSubject)
        self.cb_pause_before_addevent = QtWidgets.QCheckBox(self.tab_observations)
        self.cb_pause_before_addevent.setObjectName("cb_pause_before_addevent")
        self.verticalLayout.addWidget(self.cb_pause_before_addevent)
        spacerItem1 = QtWidgets.QSpacerItem(20, 391, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout.addItem(spacerItem1)
        self.tabWidget.addTab(self.tab_observations, "")
        self.tab_ffmpeg = QtWidgets.QWidget()
        self.tab_ffmpeg.setObjectName("tab_ffmpeg")
        self.verticalLayout_4 = QtWidgets.QVBoxLayout(self.tab_ffmpeg)
        self.verticalLayout_4.setObjectName("verticalLayout_4")
        self.verticalLayout_3 = QtWidgets.QVBoxLayout()
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.lbFFmpegPath = QtWidgets.QLabel(self.tab_ffmpeg)
        self.lbFFmpegPath.setScaledContents(False)
        self.lbFFmpegPath.setWordWrap(True)
        self.lbFFmpegPath.setObjectName("lbFFmpegPath")
        self.verticalLayout_3.addWidget(self.lbFFmpegPath)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.verticalLayout_3.addLayout(self.horizontalLayout)
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.lbFFmpegCacheDir = QtWidgets.QLabel(self.tab_ffmpeg)
        self.lbFFmpegCacheDir.setObjectName("lbFFmpegCacheDir")
        self.horizontalLayout_3.addWidget(self.lbFFmpegCacheDir)
        self.leFFmpegCacheDir = QtWidgets.QLineEdit(self.tab_ffmpeg)
        self.leFFmpegCacheDir.setObjectName("leFFmpegCacheDir")
        self.horizontalLayout_3.addWidget(self.leFFmpegCacheDir)
        self.pbBrowseFFmpegCacheDir = QtWidgets.QPushButton(self.tab_ffmpeg)
        self.pbBrowseFFmpegCacheDir.setObjectName("pbBrowseFFmpegCacheDir")
        self.horizontalLayout_3.addWidget(self.pbBrowseFFmpegCacheDir)
        self.verticalLayout_3.addLayout(self.horizontalLayout_3)
        spacerItem2 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_3.addItem(spacerItem2)
        self.verticalLayout_4.addLayout(self.verticalLayout_3)
        self.tabWidget.addTab(self.tab_ffmpeg, "")
        self.tab_spectro = QtWidgets.QWidget()
        self.tab_spectro.setObjectName("tab_spectro")
        self.verticalLayout_8 = QtWidgets.QVBoxLayout(self.tab_spectro)
        self.verticalLayout_8.setObjectName("verticalLayout_8")
        self.horizontalLayout_7 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_7.setObjectName("horizontalLayout_7")
        self.label_7 = QtWidgets.QLabel(self.tab_spectro)
        self.label_7.setObjectName("label_7")
        self.horizontalLayout_7.addWidget(self.label_7)
        self.cbSpectrogramColorMap = QtWidgets.QComboBox(self.tab_spectro)
        self.cbSpectrogramColorMap.setObjectName("cbSpectrogramColorMap")
        self.horizontalLayout_7.addWidget(self.cbSpectrogramColorMap)
        self.verticalLayout_8.addLayout(self.horizontalLayout_7)
        self.horizontalLayout_10 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_10.setObjectName("horizontalLayout_10")
        self.label_12 = QtWidgets.QLabel(self.tab_spectro)
        self.label_12.setObjectName("label_12")
        self.horizontalLayout_10.addWidget(self.label_12)
        self.sb_time_interval = QtWidgets.QSpinBox(self.tab_spectro)
        self.sb_time_interval.setMinimum(2)
        self.sb_time_interval.setMaximum(360)
        self.sb_time_interval.setProperty("value", 10)
        self.sb_time_interval.setObjectName("sb_time_interval")
        self.horizontalLayout_10.addWidget(self.sb_time_interval)
        self.verticalLayout_8.addLayout(self.horizontalLayout_10)
        spacerItem3 = QtWidgets.QSpacerItem(20, 319, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_8.addItem(spacerItem3)
        self.tabWidget.addTab(self.tab_spectro, "")
        self.tab_colors = QtWidgets.QWidget()
        self.tab_colors.setObjectName("tab_colors")
        self.verticalLayout_10 = QtWidgets.QVBoxLayout(self.tab_colors)
        self.verticalLayout_10.setObjectName("verticalLayout_10")
        self.horizontalLayout_12 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_12.setObjectName("horizontalLayout_12")
        self.verticalLayout_6 = QtWidgets.QVBoxLayout()
        self.verticalLayout_6.setObjectName("verticalLayout_6")
        self.label_10 = QtWidgets.QLabel(self.tab_colors)
        self.label_10.setOpenExternalLinks(True)
        self.label_10.setObjectName("label_10")
        self.verticalLayout_6.addWidget(self.label_10)
        self.te_behav_colors = QtWidgets.QPlainTextEdit(self.tab_colors)
        self.te_behav_colors.setObjectName("te_behav_colors")
        self.verticalLayout_6.addWidget(self.te_behav_colors)
        self.pb_reset_behav_colors = QtWidgets.QPushButton(self.tab_colors)
        self.pb_reset_behav_colors.setObjectName("pb_reset_behav_colors")
        self.verticalLayout_6.addWidget(self.pb_reset_behav_colors)
        self.horizontalLayout_12.addLayout(self.verticalLayout_6)
        self.verticalLayout_9 = QtWidgets.QVBoxLayout()
        self.verticalLayout_9.setObjectName("verticalLayout_9")
        self.label_11 = QtWidgets.QLabel(self.tab_colors)
        self.label_11.setOpenExternalLinks(True)
        self.label_11.setObjectName("label_11")
        self.verticalLayout_9.addWidget(self.label_11)
        self.te_category_colors = QtWidgets.QPlainTextEdit(self.tab_colors)
        self.te_category_colors.setObjectName("te_category_colors")
        self.verticalLayout_9.addWidget(self.te_category_colors)
        self.pb_reset_category_colors = QtWidgets.QPushButton(self.tab_colors)
        self.pb_reset_category_colors.setObjectName("pb_reset_category_colors")
        self.verticalLayout_9.addWidget(self.pb_reset_category_colors)
        self.horizontalLayout_12.addLayout(self.verticalLayout_9)
        self.verticalLayout_10.addLayout(self.horizontalLayout_12)
        self.tabWidget.addTab(self.tab_colors, "")
        self.tab_interface = QtWidgets.QWidget()
        self.tab_interface.setObjectName("tab_interface")
        self.verticalLayout_7 = QtWidgets.QVBoxLayout(self.tab_interface)
        self.verticalLayout_7.setObjectName("verticalLayout_7")
        self.formLayout = QtWidgets.QFormLayout()
        self.formLayout.setObjectName("formLayout")
        self.label_9 = QtWidgets.QLabel(self.tab_interface)
        self.label_9.setObjectName("label_9")
        self.formLayout.setWidget(0, QtWidgets.QFormLayout.LabelRole, self.label_9)
        self.sb_toolbar_icon_size = QtWidgets.QSpinBox(self.tab_interface)
        self.sb_toolbar_icon_size.setMinimum(12)
        self.sb_toolbar_icon_size.setMaximum(128)
        self.sb_toolbar_icon_size.setProperty("value", 24)
        self.sb_toolbar_icon_size.setObjectName("sb_toolbar_icon_size")
        self.formLayout.setWidget(0, QtWidgets.QFormLayout.FieldRole, self.sb_toolbar_icon_size)
        self.verticalLayout_7.addLayout(self.formLayout)
        self.cb_darkmode = QtWidgets.QCheckBox(self.tab_interface)
        self.cb_darkmode.setObjectName("cb_darkmode")
        self.verticalLayout_7.addWidget(self.cb_darkmode)
        spacerItem4 = QtWidgets.QSpacerItem(20, 386, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_7.addItem(spacerItem4)
        self.tabWidget.addTab(self.tab_interface, "")
        self.verticalLayout_2.addWidget(self.tabWidget)
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        spacerItem5 = QtWidgets.QSpacerItem(241, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_2.addItem(spacerItem5)
        self.pb_refresh = QtWidgets.QPushButton(prefDialog)
        self.pb_refresh.setObjectName("pb_refresh")
        self.horizontalLayout_2.addWidget(self.pb_refresh)
        self.pbCancel = QtWidgets.QPushButton(prefDialog)
        self.pbCancel.setObjectName("pbCancel")
        self.horizontalLayout_2.addWidget(self.pbCancel)
        self.pbOK = QtWidgets.QPushButton(prefDialog)
        self.pbOK.setObjectName("pbOK")
        self.horizontalLayout_2.addWidget(self.pbOK)
        self.verticalLayout_2.addLayout(self.horizontalLayout_2)
        self.gridLayout.addLayout(self.verticalLayout_2, 0, 0, 1, 1)

        self.retranslateUi(prefDialog)
        self.tabWidget.setCurrentIndex(5)
        QtCore.QMetaObject.connectSlotsByName(prefDialog)

    def retranslateUi(self, prefDialog):
        _translate = QtCore.QCoreApplication.translate
        prefDialog.setWindowTitle(_translate("prefDialog", "Preferences"))
        self.label.setText(_translate("prefDialog", "Default project time format"))
        self.cbTimeFormat.setItemText(0, _translate("prefDialog", "seconds"))
        self.cbTimeFormat.setItemText(1, _translate("prefDialog", "hh:mm:ss.mss"))
        self.label_6.setText(_translate("prefDialog", "Auto-save project every (minutes)"))
        self.label_3.setText(_translate("prefDialog", "Separator for behavioural strings (events export)"))
        self.leSeparator.setText(_translate("prefDialog", "|"))
        self.cbCheckForNewVersion.setText(_translate("prefDialog", "Check for new version and news"))
        self.lb_hwdec.setText(_translate("prefDialog", "MPV player hardware video decoding"))
        self.lb_project_file_indent.setText(_translate("prefDialog", "Project file indentation type"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.tab_project), _translate("prefDialog", "Project"))
        self.label_4.setText(_translate("prefDialog", "Fast forward/backward value (seconds)"))
        self.cb_adapt_fast_jump.setText(_translate("prefDialog", "Adapt the fast forward/backward jump to playback speed"))
        self.label_5.setText(_translate("prefDialog", "Playback speed step value"))
        self.label_2.setText(_translate("prefDialog", "Time offset for video/audio reposition (seconds)"))
        self.cbConfirmSound.setText(_translate("prefDialog", "Play sound when a key is pressed"))
        self.cbCloseSameEvent.setText(_translate("prefDialog", "Close the same current event independently of modifiers"))
        self.label_8.setText(_translate("prefDialog", "Beep every (seconds)"))
        self.cb_display_subtitles.setText(_translate("prefDialog", "Display subtitles"))
        self.cbTrackingCursorAboveEvent.setText(_translate("prefDialog", "Tracking cursor above current event"))
        self.cbAlertNoFocalSubject.setText(_translate("prefDialog", "Alert if focal subject is not set"))
        self.cb_pause_before_addevent.setText(_translate("prefDialog", "Pause media before \"Add event\" command"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.tab_observations), _translate("prefDialog", "Observations"))
        self.lbFFmpegPath.setText(_translate("prefDialog", "FFmpeg path:"))
        self.lbFFmpegCacheDir.setText(_translate("prefDialog", "FFmpeg cache directory"))
        self.pbBrowseFFmpegCacheDir.setText(_translate("prefDialog", "..."))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.tab_ffmpeg), _translate("prefDialog", "FFmpeg framework"))
        self.label_7.setText(_translate("prefDialog", "Spectrogram color map"))
        self.label_12.setText(_translate("prefDialog", "Default time interval (s)"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.tab_spectro), _translate("prefDialog", "Spectrogram/Wave form"))
        self.label_10.setText(_translate("prefDialog", "<html><head/><body><p>List of colors for behaviors. See <a href=\"https://matplotlib.org/api/colors_api.html\"><span style=\" text-decoration: underline; color:#0000ff;\">matplotlib colors</span></a></p></body></html>"))
        self.pb_reset_behav_colors.setText(_translate("prefDialog", "Reset colors to default"))
        self.label_11.setText(_translate("prefDialog", "<html><head/><body><p>List of colors for behavioral categories. See <a href=\"https://matplotlib.org/api/colors_api.html\"><span style=\" text-decoration: underline; color:#0000ff;\">matplotlib colors</span></a></p></body></html>"))
        self.pb_reset_category_colors.setText(_translate("prefDialog", "Reset colors to default"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.tab_colors), _translate("prefDialog", "Plot colors"))
        self.label_9.setText(_translate("prefDialog", "Toolbar icons size"))
        self.cb_darkmode.setText(_translate("prefDialog", "Dark mode"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.tab_interface), _translate("prefDialog", "Interface"))
        self.pb_refresh.setText(_translate("prefDialog", "Refresh"))
        self.pbCancel.setText(_translate("prefDialog", "Cancel"))
        self.pbOK.setText(_translate("prefDialog", "OK"))



"""
BORIS
Behavioral Observation Research Interactive Software
Copyright 2012-2024 Olivier Friard

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
  MA 02110-1301, USA.
"""

from PyQt5.QtCore import pyqtSignal, QRect, QEvent, Qt
from PyQt5.QtWidgets import QGridLayout, QPushButton, QHBoxLayout, QWidget

from . import config as cfg
from . import utilities as util


class SubjectsPad(QWidget):
    clickSignal = pyqtSignal(str)
    sendEventSignal = pyqtSignal(QEvent)
    close_signal = pyqtSignal(QRect)

    def __init__(self, pj, filtered_subjects, parent=None):
        super(SubjectsPad, self).__init__(parent)
        self.pj = pj
        self.filtered_subjects = filtered_subjects

        self.setWindowTitle("Subjects pad")
        self.grid = QGridLayout(self)
        self.installEventFilter(self)
        self.compose()

    def compose(self):
        for i in reversed(range(self.grid.count())):
            self.grid.itemAt(i).widget().setParent(None)

        subjects_list = [
            ["", self.pj[cfg.SUBJECTS][x]["name"]]
            for x in util.sorted_keys(self.pj[cfg.SUBJECTS])
            if self.pj[cfg.SUBJECTS][x]["name"] in self.filtered_subjects
        ]
        dim = int(len(subjects_list) ** 0.5 + 0.999)

        c = 0
        for i in range(1, dim + 1):
            for j in range(1, dim + 1):
                if c >= len(subjects_list):
                    break
                self.addWidget(subjects_list[c][1], i, j)
                c += 1

    def addWidget(self, subject, i, j):
        self.grid.addWidget(Button(), i, j)
        index = self.grid.count() - 1
        widget = self.grid.itemAt(index).widget()

        if widget is not None:
            widget.pushButton.setText(subject)
            color = "cyan"
            widget.pushButton.setStyleSheet(
                (
                    "background-color: {}; border-radius: 0px; min-width: 50px; max-width: 200px;"
                    " min-height:50px; max-height:200px; font-weight: bold;"
                ).format(color)
            )
            widget.pushButton.clicked.connect(lambda: self.click(subject))

    def click(self, subject):
        self.clickSignal.emit(subject)

    def eventFilter(self, receiver, event):
        """
        send event (if keypress) to main window
        """
        if event.type() == QEvent.KeyPress:
            self.sendEventSignal.emit(event)
            return True
        else:
            return False

    def closeEvent(self, event):
        """
        send event for widget geometry memory
        """
        self.close_signal.emit(self.geometry())


class Button(QWidget):
    def __init__(self, parent=None):
        super(Button, self).__init__(parent)
        self.pushButton = QPushButton()
        self.pushButton.setFocusPolicy(Qt.NoFocus)
        layout = QHBoxLayout()
        layout.addWidget(self.pushButton)
        self.setLayout(layout)



"""
module for testing project_functions.py


pytest -s -vv test_project_functions.py
"""

import pytest
import os
import sys
import json
from decimal import Decimal

sys.path.insert(0, os.path.abspath(os.path.join(os.path.dirname(__file__), '..')))

from boris import project_functions
from boris import config

@pytest.fixture()
def before():
    os.system("rm -rf output")
    os.system("mkdir output")


class Test_behavior_category(object):

    def test_1(self):
        pj = json.loads(open("files/test.boris").read())
        assert project_functions.behavior_category(pj[config.ETHOGRAM]) == {'p': '', 's': '', 'q': '', 'r': '', 'm': ''}


class Test_check_coded_behaviors(object):

    def test_1(self):
        pj = json.loads(open("files/test.boris").read())
        assert project_functions.check_coded_behaviors(pj) == set()


class Test_check_if_media_available(object):

    def test_media_not_available(self):
        pj = json.loads(open("files/test.boris").read())

        assert project_functions.check_if_media_available(pj[config.OBSERVATIONS]["offset positif"],
                                                          'files/test.boris') == (False, 'Media file <b>video_test_25fps_360s.mp4</b> not found')

    def test_live_observation(self):
        pj = json.loads(open("files/test.boris").read())

        assert project_functions.check_if_media_available(pj[config.OBSERVATIONS]["live"],
                                                          'files/test.boris') == (True, "")

    '''
    def test_media_available(self):
        pj = json.loads(open("files/test_without_media_files_paths.boris").read())

        assert project_functions.check_if_media_available(pj[OBSERVATIONS]["geese1"],
                                                          'files/test.boris') == (True, "")
    '''



class Test_check_project_integrity(object):

    def test_observation_not_paired(self):
        """
        one observation not paired
        """
        pj = json.loads(open("files/test.boris").read())

        results = project_functions.check_project_integrity(pj,
                                                            config.HHMMSS,
                                                           'files/test.boris',
                                                            media_file_available=False)

        assert results == '''Observation: <b>live not paired</b><br>The behavior <b>s</b>  is not PAIRED for subject "<b>No focal subject</b>" at <b>00:00:26.862</b><br>'''


    def test_modifiers_with_trailing_spaces(self):
        """
        Project containing some modifiers with trailing spaces
        """

        pj = json.loads(open("files/test_with_leading_trailing_spaces_in_modifiers.boris").read())

        results = project_functions.check_project_integrity(pj,
                                                            config.HHMMSS,
                                                           "files/test_with_leading_trailing_spaces_in_modifiers.boris",
                                                            media_file_available=False)

        #print(results)
        #assert results == '''The following modifier defined in ethogram has leading/trailing spaces: <b>a&#9608;&#9608;&#9608;</b><br><br>The following modifier defined in ethogram has leading/trailing spaces: <b>c&#9608;&#9608;</b><br><br>The following modifier defined in ethogram has leading/trailing spaces: <b>c&#9608;</b><br><br>The following modifier defined in ethogram has leading/trailing spaces: <b>d&#9608;&#9608;</b>'''
        assert results == '''The following <b>modifier</b> defined in ethogram has leading/trailing spaces or special chars: <b>a&#9608;&#9608;&#9608;</b><br><br>The following <b>modifier</b> defined in ethogram has leading/trailing spaces or special chars: <b>c&#9608;&#9608;</b><br><br>The following <b>modifier</b> defined in ethogram has leading/trailing spaces or special chars: <b>c&#9608;</b><br><br>The following <b>modifier</b> defined in ethogram has leading/trailing spaces or special chars: <b>d&#9608;&#9608;</b>'''

class Test_check_state_events_obs(object):

    def test_observation_ok(self):
        pj = json.loads(open("files/test.boris").read())

        results = project_functions.check_state_events_obs("offset positif",
                                                            pj[config.ETHOGRAM],
                                                            pj[config.OBSERVATIONS]["offset positif"],
                                                            config.HHMMSS)
        # print(results)

        assert results == (True, 'No problem detected')


    def test_observation_not_paired(self):
        pj = json.loads(open("files/test.boris").read())

        results = project_functions.check_state_events_obs("live not paired",
                                                            pj[config.ETHOGRAM],
                                                            pj[config.OBSERVATIONS]["live not paired"],
                                                            config.HHMMSS)
        # print(results)

        assert results == (False, 'The behavior <b>s</b>  is not PAIRED for subject "<b>No focal subject</b>" at <b>00:00:26.862</b><br>')


class Test_export_observations_list(object):

    @pytest.mark.usefixtures("before")
    def test1(self):
        pj = json.loads(open("files/test2.boris").read())
        selected_observations = [x for x in pj[config.OBSERVATIONS]]

        result = project_functions.export_observations_list(pj=pj,
                                                            file_name="output/export_observations_list_test1.tsv",
                                                            selected_observations = selected_observations,
                                                            output_format="tsv"
                                                            )
        assert result == True
        assert open("files/export_observations_list_test1.tsv").read() == open("output/export_observations_list_test1.tsv").read()


class Test_media_full_path(object):

    def test_file_and_dir(self):
        assert project_functions.media_full_path("geese1.mp4", os.getcwd() + "/files/test.boris") == os.getcwd() + "/files/geese1.mp4"

    def test_file_not_found(self):
        assert project_functions.media_full_path("geese1.xxx", os.getcwd() + "/files/test.boris") == ""


    def test_project_file_not_found(self):
        assert project_functions.media_full_path("geese1.xxx", os.getcwd() + "/files/test.xxx.boris") == ""



class Test_remove_media_files_path(object):

    def test_1(self):
        """
        test the deletion of the media files path in project
        """

        pj = json.loads(open("files/test2.boris").read())
        pj_wo_media_files_paths = project_functions.remove_media_files_path(pj)

        assert pj_wo_media_files_paths == json.loads(open("files/test_without_media_files_paths.boris").read())







'''

def test_observation_total_length1():
    out = project_functions.observation_total_length(pj[OBSERVATIONS]["live"])
    assert out == Decimal("26.63")
    return out

def test_observation_total_length2():
    out = project_functions.observation_total_length(pj[OBSERVATIONS]["observation #1"])
    assert out == Decimal("225.6399999999999863575794734")
    return out


for f in [test_remove_media_files_path,
          test_media_full_path1,
          test_observation_total_length1,
          test_observation_total_length2,
          ]:
    print(f)
    print(f())
    print("=====================================")

'''



# -*- coding: utf-8 -*-
"""
Utilities to process and convert svg images to png using palette colors.
"""

# Standard library imports
import logging
import os
import re
import sys
import tempfile
import subprocess

# Third party imports
from qtpy.QtCore import QSize
from qtpy.QtGui import QIcon
from qtpy.QtWidgets import QApplication

# Local imports
from qdarkstyle import (IMAGES_PATH, PACKAGE_PATH, QRC_FILE_SUFFIX, QSS_FILE_SUFFIX, QSS_PATH,
                        STYLES_SCSS_FILE, SVG_PATH)


IMAGE_BLACKLIST = ['base_palette']

TEMPLATE_QRC_HEADER = '''
<RCC warning="WARNING! File created programmatically. All changes made in this file will be lost!">
  <qresource prefix="{resource_prefix}">
'''

TEMPLATE_QRC_FILE = '    <file>rc/{fname}</file>'

TEMPLATE_QRC_FOOTER = '''
  </qresource>
  <qresource prefix="{style_prefix}">
      <file>{qss_file}</file>
  </qresource>
</RCC>
'''

_logger = logging.getLogger(__name__)


def _get_file_color_map(fname, palette):
    """
    Return map of files (i.e states) to color from given palette.
    """
    color_disabled = palette.COLOR_BACKGROUND_4
    color_focus = palette.COLOR_ACCENT_5
    color_pressed = palette.COLOR_ACCENT_2
    color_normal = palette.COLOR_TEXT_1

    name, ext = fname.split('.')
    file_colors = {
        fname: color_normal,
        name + '_disabled.' + ext: color_disabled,
        name + '_focus.' + ext: color_focus,
        name + '_pressed.' + ext: color_pressed,
    }

    return file_colors


def _create_colored_svg(svg_path, temp_svg_path, color):
    """
    Replace base svg with fill color.
    """
    with open(svg_path, 'r') as fh:
        data = fh.read()

    base_color = '#ff0000'  # Hardcoded in base svg files
    new_data = data.replace(base_color, color)

    with open(temp_svg_path, 'w') as fh:
        fh.write(new_data)


def convert_svg_to_png(svg_path, png_path, height, width):
    """
    Convert svg files to png files using Qt.
    """
    size = QSize(height, width)
    icon = QIcon(svg_path)
    pixmap = icon.pixmap(size)
    img = pixmap.toImage()
    dirname = os.path.dirname(png_path)

    if not os.path.exists(dirname):
        os.makedirs(dirname)

    img.save(os.path.abspath(png_path))


def create_palette_image(base_svg_path=SVG_PATH, path=IMAGES_PATH,
                         palette=None):
    """
    Create palette image svg and png image on specified path.
    """
    # Needed to use QPixmap
    _ = QApplication([])

    if palette is None:
        _logger.error("Please pass a palette class in order to create its "
                      "associated images")
        sys.exit(1)

    if palette.ID is None:
        _logger.error("A QDarkStyle palette requires an ID!")
        sys.exit(1)

    base_palette_svg_path = os.path.join(base_svg_path, 'base_palette.svg')
    palette_svg_path = os.path.join(path, palette.ID, 'palette.svg')
    palette_png_path = os.path.join(path, palette.ID, 'palette.png')

    _logger.info("Creating palette image ...")
    _logger.info(f"Base SVG: {base_palette_svg_path}")
    _logger.info(f"To SVG: {palette_svg_path}")
    _logger.info(f"To PNG: {palette_png_path}")

    with open(base_palette_svg_path, 'r') as fh:
        data = fh.read()

    color_palette = palette.color_palette()

    for color_name, color_value in color_palette.items():
        data = data.replace('{{ ' + color_name + ' }}', color_value.lower())

    with open(palette_svg_path, 'w+') as fh:
        fh.write(data)

    convert_svg_to_png(palette_svg_path, palette_png_path, 4000, 4000)

    return palette_svg_path, palette_png_path


def create_images(base_svg_path=SVG_PATH, rc_path=None, palette=None):
    """Create resources `rc` png image files from base svg files and palette.

    Search all SVG files in `base_svg_path` excluding IMAGE_BLACKLIST,
    change its colors using `palette` creating temporary SVG files, for each
    state generating PNG images for each size `heights`.

    Args:
        base_svg_path (str, optional): [description]. Defaults to SVG_PATH.
        rc_path (str, optional): [description].
        palette (Palette, optional): Palette.
    """

    # Needed to use QPixmap
    _ = QApplication([])

    if palette is None:
        _logger.error("Please pass a palette class in order to create its "
                      "associated file")
        sys.exit(1)

    if palette.ID is None:
        _logger.error("A QDarkStyle palette requires an ID!")
        sys.exit(1)

    if not rc_path:
        rc_path = os.path.join(PACKAGE_PATH, palette.ID, 'rc')

    temp_dir = tempfile.mkdtemp()
    svg_fnames = [f for f in os.listdir(base_svg_path) if f.endswith('.svg')]
    base_height = 32

    # See: https://doc.qt.io/qt-5/scalability.html
    heights = {
        32: '.png',
        64: '@2x.png',
    }

    _logger.info("Creating images ...")
    _logger.info(f"SVG folder: {base_svg_path}")
    _logger.info(f"TMP folder: {temp_dir}")
    _logger.info(f"PNG folder: {rc_path}")

    num_svg = len(svg_fnames)
    num_png = 0
    num_ignored = 0
    num_ignored_list = []

    # Get rc links from scss to check matches
    rc_list = get_rc_links_from_scss()
    num_rc_list = len(rc_list)

    for height, ext in heights.items():
        width = height

        _logger.debug(f" Size HxW (px): {height} X {width}")

        for svg_fname in svg_fnames:
            svg_name = svg_fname.split('.')[0]

            # Skip blacklist
            if svg_name not in IMAGE_BLACKLIST:
                svg_path = os.path.join(base_svg_path, svg_fname)
                color_files = _get_file_color_map(svg_fname, palette=palette)

                _logger.log(logging.NOTSET, f"  Working on: {os.path.basename(svg_fname)}")

                # Replace colors and create all file for different states
                for color_svg_name, color in color_files.items():
                    temp_svg_path = os.path.join(temp_dir, color_svg_name)
                    _create_colored_svg(svg_path, temp_svg_path, color)

                    png_fname = color_svg_name.replace('.svg', ext)
                    png_path = os.path.join(rc_path, png_fname)
                    convert_svg_to_png(temp_svg_path, png_path, height, width)
                    num_png += 1
                    _logger.log(logging.NOTSET, f"   Creating: {os.path.basename(png_fname)}")

                    # Check if the rc_name is in the rc_list from scss
                    # only for the base size
                    if height == base_height:
                        rc_base = os.path.basename(rc_path)
                        png_base = os.path.basename(png_fname)
                        rc_name = '/' + rc_base + '/' + png_base
                        try:
                            rc_list.remove(rc_name)
                        except ValueError:
                            pass
            else:
                num_ignored += 1
                _logger.debug(f"  Ignored blacklist: {os.path.basename(svg_fname)}")
                num_ignored_list.append(svg_fname)

    _logger.info(f"# SVG files: {num_svg}")
    _logger.info(f"# SVG ignored: {num_ignored}")
    _logger.info(f"SVG ignored: {num_ignored_list}")
    _logger.info(f"# PNG files: {num_png}")
    _logger.info(f"# RC links: {num_rc_list}")
    _logger.info(f"# RC links in _style.scss not in RC: {len(rc_list)}")
    _logger.info(f"RC links in _style.scss not in RC: {rc_list}")


def generate_qrc_file(resource_prefix='qss_icons', style_prefix='qdarkstyle',
                      palette=None):
    """
    Generate the QRC file programmatically.

    Search all RC folder for PNG images and create a QRC file.

    Args:
        resource_prefix (str, optional): Prefix used in resources.
            Defaults to 'qss_icons'.
        style_prefix (str, optional): Prefix used to this style.
            Defaults to 'qdarkstyle'.
        palette (Palette, optional): Palette.
    """

    files = []

    if palette is None:
        _logger.error("Please pass a palette class in order to create its "
                      "qrc file")
        sys.exit(1)

    if palette.ID is None:
        _logger.error("A QDarkStyle palette requires an ID!")
        sys.exit(1)

    palette_path = os.path.join(PACKAGE_PATH, palette.ID)
    rc_path = os.path.join(palette_path, 'rc')
    qss_file = palette.ID + QSS_FILE_SUFFIX
    qrc_file = palette.ID + QRC_FILE_SUFFIX
    qrc_filepath = os.path.join(palette_path, qrc_file)
    resource_prefix = resource_prefix + '/' + palette.ID
    style_prefix = style_prefix + '/' + palette.ID

    _logger.info("Generating QRC file ...")
    _logger.info(f"Resource prefix: {resource_prefix}")
    _logger.info(f"Style prefix: {style_prefix}")

    _logger.info(f"Searching in: {rc_path}")

    # Search by png images
    for fname in sorted(os.listdir(rc_path)):
        if os.path.splitext(fname)[1] == '.png':
            files.append(TEMPLATE_QRC_FILE.format(fname=fname))

    # Join parts
    qrc_content = (TEMPLATE_QRC_HEADER.format(resource_prefix=resource_prefix)
                   + '\n'.join(files)
                   + TEMPLATE_QRC_FOOTER.format(style_prefix=style_prefix, qss_file=qss_file))

    _logger.info(f"Writing in: {qrc_filepath}")

    # Write qrc file
    with open(qrc_filepath, 'w') as fh:
        fh.write(qrc_content)


def get_rc_links_from_scss(pattern=r"\/rc.*\.png"):
    """
    Get all rc links from scss file returning the list of unique links.

    Args:
        pattern (str): regex pattern to find the links.

    Returns:
        list(str): list of unique links found.
    """

    style_scss_filepath = os.path.join(QSS_PATH, STYLES_SCSS_FILE)

    with open(style_scss_filepath, 'r') as fh:
        data = fh.read()

    lines = data.split("\n")
    compiled_exp = re.compile('(' + pattern + ')')

    rc_list = []

    for line in lines:
        match = re.search(compiled_exp, line)
        if match:
            path = match.group(1)
            rc_list.append(match.group(1))

    rc_list = list(set(rc_list))

    return rc_list


def compile_qrc_file(compile_for='qtpy', qrc_path=None, palette=None):
    """
    Compile the QRC file converting it to _rc.py nad/or .rcc.

    When using an abstraction layer (QtPy/pyqtgraph) over a binging
    (PySide/PyQt), in the end, it changes the importing name.

    For all other `compile_for` that not 'qtpy', it prefixes the file name
    with `compile_for` value.

    Args:
        compile_for (list, optional): Prefix used in resources.
            Defaults to 'qtpy'. Possible values are 'qtpy', 'pyqtgraph',
            'pyqt', 'pyqt5', 'pyside', 'pyside2', 'qt', 'qt5', 'all'.
        qrc_path (str, optional): .qrc folder path.
            Defaults to None.
        palette (Palette, optional): Palette.
    """

    if palette is None:
        _logger.error("Please pass a palette class in order to create its "
                      "associated file")
        sys.exit(1)

    if palette.ID is None:
        _logger.error("A QDarkStyle palette requires an ID!")
        sys.exit(1)

    if not qrc_path:
        qrc_path = os.path.join(PACKAGE_PATH, palette.ID)

    qrc_file = palette.ID + QRC_FILE_SUFFIX

    # get name without extension
    filename = os.path.splitext(qrc_file)[0]

    ext = '_rc.py'
    ext_c = '.rcc'

    # creating names
    py_file_pyqt6 = 'pyqt6_' + filename + ext
    py_file_pyqt5 = 'pyqt5_' + filename + ext
    py_file_pyqt = 'pyqt_' + filename + ext
    py_file_pyside6 = 'pyside6_' + filename + ext
    py_file_pyside2 = 'pyside2_' + filename + ext
    py_file_pyside = 'pyside_' + filename + ext
    py_file_qtpy = '' + filename + ext
    py_file_pyqtgraph = 'pyqtgraph_' + filename + ext

    # it is simple to change the directory, otherwise we need to add
    # more arguments for each compiler
    old_cwd = os.getcwd()
    os.chdir(qrc_path)

    # Shell kwarg to pass to subprocess
    shell = True if os.name == 'nt' else False

    # calling external commands
    if compile_for in ['pyqt', 'pyqtgraph', 'all']:
        _logger.info("Compiling using PyQt4 ...")
        try:
            subprocess.call(['pyrcc4', '-py3', qrc_file, '-o', py_file_pyqt], shell=shell)
        except FileNotFoundError:
            _logger.error("You must install pyrcc4")

    if compile_for in ['pyqt5', 'qtpy','all']:
        _logger.info("Compiling using PyQt5 ...")
        try:
            subprocess.call(['pyrcc5', qrc_file, '-o', py_file_pyqt5], shell=shell)
        except FileNotFoundError:
            _logger.error("You must install pyrcc5")

    if compile_for in ['pyside', 'all']:
        _logger.info("Compiling using PySide ...")
        try:
            subprocess.call(['pyside-rcc', '-py3', qrc_file, '-o', py_file_pyside], shell=shell)
        except FileNotFoundError:
            _logger.error("You must install pyside-rcc")

    if compile_for in ['pyside2', 'all']:
        _logger.info("Compiling using PySide 2...")
        try:
            subprocess.call(['pyside2-rcc', qrc_file, '-o', py_file_pyside2], shell=shell)
        except FileNotFoundError:
            _logger.error("You must install pyside2-rcc")

    if compile_for in ['pyside6', 'all']:
        _logger.info("Compiling using PySide 6...")
        try:
            subprocess.call(['pyside6-rcc', '-g', 'python', qrc_file, '-o', py_file_pyside6], shell=shell)
        except FileNotFoundError:
            _logger.error("You must install pyside6-rcc")

    if compile_for in ['qtpy', 'all']:
        _logger.info("Converting for QtPy ...")
        # special case - qtpy - syntax is PyQt5
        with open(py_file_pyqt5, 'r') as file:
            filedata = file.read()

        # replace the target string
        filedata = filedata.replace('from PyQt5', 'from qtpy')

        with open(py_file_qtpy, 'w+') as file:
            # write the file out again
            file.write(filedata)

        if compile_for not in ['pyqt5']:
            os.remove(py_file_pyqt5)

    if compile_for in ['pyqtgraph', 'all']:
        _logger.info("Converting for PyQtGraph ...")
        # special case - pyqtgraph - syntax is PyQt4
        with open(py_file_pyqt, 'r') as file:
            filedata = file.read()

        # replace the target string
        filedata = filedata.replace('from PyQt4', 'from pyqtgraph.Qt')

        with open(py_file_pyqtgraph, 'w+') as file:
            # write the file out again
            file.write(filedata)

    os.chdir(old_cwd)



"""
module for testing observation GUI


pytest -s -vv test_observation_gui.py
"""


import sys
import os
from PyQt5.QtCore import *

sys.path.insert(0, os.path.abspath(os.path.join(os.path.dirname(__file__), '..')))

from boris import preferences
from boris import config


def test_ok(qtbot):

    w = preferences.Preferences()

    qtbot.addWidget(w)
    
    def handle_dialog():
        qtbot.mouseClick(w.pbOK, Qt.LeftButton)

    QTimer.singleShot(1000, handle_dialog)

    r = w.exec_()
    assert r == 1


def test_cancel(qtbot):

    w = preferences.Preferences()

    qtbot.addWidget(w)
    
    def handle_dialog():
        qtbot.mouseClick(w.pbCancel, Qt.LeftButton)

    QTimer.singleShot(1000, handle_dialog)
    
    r = w.exec_()
    assert r == 0


def test_preferences_functions(qtbot):

    w = preferences.Preferences()
    w.rb_frames_mem_disk()
    w.reset_colors()

    qtbot.addWidget(w)
    
    def handle_dialog():
        qtbot.mouseClick(w.pbOK, Qt.LeftButton)

    QTimer.singleShot(1000, handle_dialog)

    r = w.exec_()
    assert r == 1

'''
def test_refresh_preferences(qtbot):

    w = preferences.Preferences()

    qtbot.addWidget(w)
    
    def handle_dialog():
        qtbot.keyClick(w.qm, Qt.Key_Enter)

    QTimer.singleShot(1000, handle_dialog)
    w.refresh_preferences()

    r = w.exec_()
    assert w.flag_refresh == True
'''




# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'add_modifier.ui'
#
# Created by: PyQt5 UI code generator 5.15.7
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_Dialog(object):
    def setupUi(self, Dialog):
        Dialog.setObjectName("Dialog")
        Dialog.resize(638, 636)
        self.verticalLayout_5 = QtWidgets.QVBoxLayout(Dialog)
        self.verticalLayout_5.setObjectName("verticalLayout_5")
        self.verticalLayout_4 = QtWidgets.QVBoxLayout()
        self.verticalLayout_4.setObjectName("verticalLayout_4")
        self.horizontalLayout_5 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_5.setObjectName("horizontalLayout_5")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout()
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.lbModifier = QtWidgets.QLabel(Dialog)
        self.lbModifier.setObjectName("lbModifier")
        self.verticalLayout_2.addWidget(self.lbModifier)
        self.leModifier = QtWidgets.QLineEdit(Dialog)
        self.leModifier.setObjectName("leModifier")
        self.verticalLayout_2.addWidget(self.leModifier)
        self.lbCode = QtWidgets.QLabel(Dialog)
        self.lbCode.setObjectName("lbCode")
        self.verticalLayout_2.addWidget(self.lbCode)
        self.leCode = QtWidgets.QLineEdit(Dialog)
        self.leCode.setObjectName("leCode")
        self.verticalLayout_2.addWidget(self.leCode)
        self.lbCodeHelp = QtWidgets.QLabel(Dialog)
        self.lbCodeHelp.setWordWrap(True)
        self.lbCodeHelp.setObjectName("lbCodeHelp")
        self.verticalLayout_2.addWidget(self.lbCodeHelp)
        spacerItem = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_2.addItem(spacerItem)
        self.horizontalLayout_5.addLayout(self.verticalLayout_2)
        self.verticalLayout_3 = QtWidgets.QVBoxLayout()
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.pbAddModifier = QtWidgets.QPushButton(Dialog)
        self.pbAddModifier.setText("")
        self.pbAddModifier.setObjectName("pbAddModifier")
        self.verticalLayout_3.addWidget(self.pbAddModifier)
        self.pbModifyModifier = QtWidgets.QPushButton(Dialog)
        self.pbModifyModifier.setText("")
        self.pbModifyModifier.setObjectName("pbModifyModifier")
        self.verticalLayout_3.addWidget(self.pbModifyModifier)
        spacerItem1 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_3.addItem(spacerItem1)
        self.horizontalLayout_5.addLayout(self.verticalLayout_3)
        self.verticalLayout = QtWidgets.QVBoxLayout()
        self.verticalLayout.setObjectName("verticalLayout")
        self.tabWidgetModifiersSets = QtWidgets.QTabWidget(Dialog)
        self.tabWidgetModifiersSets.setMaximumSize(QtCore.QSize(16777215, 30))
        self.tabWidgetModifiersSets.setTabPosition(QtWidgets.QTabWidget.North)
        self.tabWidgetModifiersSets.setTabShape(QtWidgets.QTabWidget.Rounded)
        self.tabWidgetModifiersSets.setElideMode(QtCore.Qt.ElideNone)
        self.tabWidgetModifiersSets.setDocumentMode(True)
        self.tabWidgetModifiersSets.setObjectName("tabWidgetModifiersSets")
        self.verticalLayout.addWidget(self.tabWidgetModifiersSets)
        self.lb_name = QtWidgets.QLabel(Dialog)
        self.lb_name.setObjectName("lb_name")
        self.verticalLayout.addWidget(self.lb_name)
        self.le_name = QtWidgets.QLineEdit(Dialog)
        self.le_name.setObjectName("le_name")
        self.verticalLayout.addWidget(self.le_name)
        self.lb_description = QtWidgets.QLabel(Dialog)
        self.lb_description.setObjectName("lb_description")
        self.verticalLayout.addWidget(self.lb_description)
        self.le_description = QtWidgets.QLineEdit(Dialog)
        self.le_description.setObjectName("le_description")
        self.verticalLayout.addWidget(self.le_description)
        self.lbType = QtWidgets.QLabel(Dialog)
        self.lbType.setObjectName("lbType")
        self.verticalLayout.addWidget(self.lbType)
        self.cbType = QtWidgets.QComboBox(Dialog)
        self.cbType.setObjectName("cbType")
        self.cbType.addItem("")
        self.cbType.addItem("")
        self.cbType.addItem("")
        self.cbType.addItem("")
        self.verticalLayout.addWidget(self.cbType)
        self.lbValues = QtWidgets.QLabel(Dialog)
        self.lbValues.setObjectName("lbValues")
        self.verticalLayout.addWidget(self.lbValues)
        self.lwModifiers = QtWidgets.QListWidget(Dialog)
        self.lwModifiers.setObjectName("lwModifiers")
        self.verticalLayout.addWidget(self.lwModifiers)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.pbMoveUp = QtWidgets.QPushButton(Dialog)
        self.pbMoveUp.setObjectName("pbMoveUp")
        self.horizontalLayout.addWidget(self.pbMoveUp)
        self.pbMoveDown = QtWidgets.QPushButton(Dialog)
        self.pbMoveDown.setObjectName("pbMoveDown")
        self.horizontalLayout.addWidget(self.pbMoveDown)
        self.verticalLayout.addLayout(self.horizontalLayout)
        self.pbRemoveModifier = QtWidgets.QPushButton(Dialog)
        self.pbRemoveModifier.setObjectName("pbRemoveModifier")
        self.verticalLayout.addWidget(self.pbRemoveModifier)
        self.pb_sort_modifiers = QtWidgets.QPushButton(Dialog)
        self.pb_sort_modifiers.setObjectName("pb_sort_modifiers")
        self.verticalLayout.addWidget(self.pb_sort_modifiers)
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.pbAddSet = QtWidgets.QPushButton(Dialog)
        self.pbAddSet.setObjectName("pbAddSet")
        self.horizontalLayout_3.addWidget(self.pbAddSet)
        self.pbRemoveSet = QtWidgets.QPushButton(Dialog)
        self.pbRemoveSet.setObjectName("pbRemoveSet")
        self.horizontalLayout_3.addWidget(self.pbRemoveSet)
        self.verticalLayout.addLayout(self.horizontalLayout_3)
        self.horizontalLayout_4 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_4.setObjectName("horizontalLayout_4")
        self.pbMoveSetLeft = QtWidgets.QPushButton(Dialog)
        self.pbMoveSetLeft.setObjectName("pbMoveSetLeft")
        self.horizontalLayout_4.addWidget(self.pbMoveSetLeft)
        self.pbMoveSetRight = QtWidgets.QPushButton(Dialog)
        self.pbMoveSetRight.setObjectName("pbMoveSetRight")
        self.horizontalLayout_4.addWidget(self.pbMoveSetRight)
        self.verticalLayout.addLayout(self.horizontalLayout_4)
        self.pb_add_subjects = QtWidgets.QPushButton(Dialog)
        self.pb_add_subjects.setObjectName("pb_add_subjects")
        self.verticalLayout.addWidget(self.pb_add_subjects)
        self.pb_load_file = QtWidgets.QPushButton(Dialog)
        self.pb_load_file.setObjectName("pb_load_file")
        self.verticalLayout.addWidget(self.pb_load_file)
        spacerItem2 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout.addItem(spacerItem2)
        self.horizontalLayout_5.addLayout(self.verticalLayout)
        self.verticalLayout_4.addLayout(self.horizontalLayout_5)
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        spacerItem3 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_2.addItem(spacerItem3)
        self.pbCancel = QtWidgets.QPushButton(Dialog)
        self.pbCancel.setObjectName("pbCancel")
        self.horizontalLayout_2.addWidget(self.pbCancel)
        self.pbOK = QtWidgets.QPushButton(Dialog)
        self.pbOK.setObjectName("pbOK")
        self.horizontalLayout_2.addWidget(self.pbOK)
        self.verticalLayout_4.addLayout(self.horizontalLayout_2)
        self.verticalLayout_5.addLayout(self.verticalLayout_4)

        self.retranslateUi(Dialog)
        self.tabWidgetModifiersSets.setCurrentIndex(-1)
        QtCore.QMetaObject.connectSlotsByName(Dialog)

    def retranslateUi(self, Dialog):
        _translate = QtCore.QCoreApplication.translate
        Dialog.setWindowTitle(_translate("Dialog", "Set modifiers"))
        self.lbModifier.setText(_translate("Dialog", "Modifier"))
        self.lbCode.setText(_translate("Dialog", "Key code"))
        self.lbCodeHelp.setText(_translate("Dialog", "Key code is case sensitive. Type one character or a function key (F1, F2... F12)"))
        self.lb_name.setText(_translate("Dialog", "Set name"))
        self.lb_description.setText(_translate("Dialog", "Description"))
        self.lbType.setText(_translate("Dialog", "Modifier type"))
        self.cbType.setItemText(0, _translate("Dialog", "Single selection"))
        self.cbType.setItemText(1, _translate("Dialog", "Multiple selection"))
        self.cbType.setItemText(2, _translate("Dialog", "Numeric"))
        self.cbType.setItemText(3, _translate("Dialog", "Value from external data file"))
        self.lbValues.setText(_translate("Dialog", "Values"))
        self.pbMoveUp.setText(_translate("Dialog", "Move modifier up"))
        self.pbMoveDown.setText(_translate("Dialog", "Move modifier down"))
        self.pbRemoveModifier.setText(_translate("Dialog", "Remove modifier"))
        self.pb_sort_modifiers.setText(_translate("Dialog", "Sort modifiers"))
        self.pbAddSet.setText(_translate("Dialog", "Add set of modifiers"))
        self.pbRemoveSet.setText(_translate("Dialog", "Remove set of modifiers"))
        self.pbMoveSetLeft.setText(_translate("Dialog", "Move set left"))
        self.pbMoveSetRight.setText(_translate("Dialog", "Move set right"))
        self.pb_add_subjects.setText(_translate("Dialog", "Add subjects as modifiers"))
        self.pb_load_file.setText(_translate("Dialog", "Load modifiers from file"))
        self.pbCancel.setText(_translate("Dialog", "Cancel"))
        self.pbOK.setText(_translate("Dialog", "OK"))
